#!/usr/bin/perl

#
# vdradmin.pl by Thomas Koch <tom@linvdr.org>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
# Or, point your browser to http://www.gnu.org/copyleft/gpl.html
#
# 08.10.2001
#
# TODO: a timeout during request
#       Kanal Editor

my($BASENAME);
BEGIN {
    $0 =~ /(^.*\/)/;
    $BASENAME = $1;
    unshift(@INC, $BASENAME . "lib/");
}

use CGI qw/:standard/;
use IO::Socket;
use HTML::Template;
use Time::Local;
use POSIX;
use MIME::Base64;
use Time::HiRes qw(gettimeofday tv_interval);

use strict;

my(%CONFIG);
$CONFIG{VDRHOST}          = "video";
$CONFIG{SERVERPORT}       = 8001;
$CONFIG{VDRPORT}          = 2001;
$CONFIG{LANG}             = "de";
$CONFIG{CACHE_TIMEOUT}    = 360;
$CONFIG{PCHANL}           = 0;
$CONFIG{USER}             = "linvdr";
$CONFIG{PASS}             = "linvdr";
$CONFIG{LOGGING}          = 0;
$CONFIG{LOGFILE}          = "vdradmind.log";
$CONFIG{ZPUFA}            = 5;
$CONFIG{ZPUFE}            = 5;
my($VERSION)              = "0.4";
my($SERVERVERSION)        = "vdradmind/$VERSION";

my($TEMPLATEDIR)          = "${BASENAME}template";
my($CONFFILE)             = "${BASENAME}conffile";


$|=1;
my($LOCALLASTGET) = 0; my(@CACHE_CHAN, %EPG); 
my(%mimehash) = ( 
    html => "text/html",
    png  => "image/png",
    gif  => "image/gif",
    jpg  => "image/jpeg"
);

my($DAEMON) = 1;
for(@ARGV) {
    if(/\-h|\-\-help/) {
	print("Usage $0 [OPTION]...\n");
	print("A perl client for the Linux Video Disk Recorder.\n\n");
	print("  -nd\tdon't fork\n\n");
	print("Report bugs to <tom\@linvdr.org>.\n");
	exit(0);
    }
    if(/\-nd/) { $DAEMON = 0; last; }
    #print("Unknown parameter $_!\nTry -h\n");
}

if($DAEMON) {
    my($pid) = fork;
    if($pid != 0) {
        print("vdradmind.pl $VERSION started with pid $pid.\n");
	open(F, ">${BASENAME}vdradmind.pid"); print(F $pid); close(F);
        exit(0);
    }
}


ReadConfig();
my($Socket) = IO::Socket::INET->new( Proto => 'tcp',
				     LocalPort => $CONFIG{SERVERPORT},
				     Listen => SOMAXCONN,
				     Reuse => 1);				     
die("can't start Server $!\n") if (!$Socket);

$SIG{PIPE} = \&SignalHandler;

#############################################################################
# Mainloop
#
my($Client, $buf, $MyURL);
while($Client = $Socket->accept()) {
    my($AuthUser, $AuthPass, $Request);
    $Request = <$Client>;
    my($LogRequest) = $Request; $LogRequest =~ s/\r\n//g;
    $Request =~ s/^[A-Z]+ //;
    $Request =~ s/ [hH][Tt][Tt][pP]\/[0-9]+\.*[0-9]+[\n\r\015\013]+$//g;
    my($URI) = $Request;
    
    my($mstime0) = [gettimeofday];
    
    do {
	$Request = <$Client>;
	$Request =~ s/\r\n//g;
	
	if($Request =~ /Host: (.*)/i) { $MyURL = "http://$1"; }
	if($Request =~ /Authorization: basic (.*)/i) {
	    ($AuthUser, $AuthPass) = split(/\:/, decode_base64($1));
	}   
    } while($Request);

	
    if(!$AuthUser) {
	SendNoAuthHeader("401");
	close($Client);
	next;
    } else {
	if($CONFIG{USER} ne $AuthUser || $CONFIG{PASS} ne $AuthPass) {
	    SendNoAuthHeader("401");
	    close($Client);
	    next;
	}
    }
    
    if($URI =~ /\/vdradmin.pl\?(.*)/) {
	$MyURL .= "/vdradmin.pl";
	do_something($1);
    } elsif($URI eq "/") {
	SendFile("index.html");
    } else {
	SendFile($URI);
    }
    
    my($mstime1) = [gettimeofday];
    
    Log($Client->peerhost . ": $LogRequest " . tv_interval($mstime0, $mstime1)
	. " L:" . length($LogRequest));
    
    close($Client);
}

#############################################################################
#############################################################################
sub ReadConfig {    
    if(-e $CONFFILE) {
	open(CONF, $CONFFILE);
	while(<CONF>) {
	    chomp;
	    my($key, $value) = split(/ \= /, $_, 2);
	    $CONFIG{$key} = $value;
	}
	close(CONF);
    } else {
	open(CONF, ">$CONFFILE");
	for(keys(%CONFIG)) {
	    print(CONF "$_ = $CONFIG{$_}\n");
	}
	close(CONF);
	return(1);
    }
    return(0);
}   


sub SignalHandler {
    my($sig) = @_;
    #print("SIG$sig received!\n");
    #Log("SIG$sig received!");
}


sub Log {
    if($CONFIG{LOGGING}) {
	open(LOGFILE, ">>" . $CONFIG{LOGFILE});
	my($text) = @_;
	print LOGFILE (strftime("%d.%m %H:%M:%S", localtime(time)) . " $text\n");	
	print(strftime("%d.%m %H:%M:%S", localtime(time)) . " $text\n");	
	close(LOGFILE);
    }
}


sub SVDRPSend {
    my($cmd) = @_; my($error) = 0;
    my($Socket) = IO::Socket::INET->new(
	PeerAddr => $CONFIG{VDRHOST}, 
	PeerPort => $CONFIG{VDRPORT}, 
	Proto => 'tcp') || ($error = 1);
	
    if($error) {
	HTMLError("can't connect to host \"$CONFIG{VDRHOST}\" port $CONFIG{VDRPORT}!");
	return(1);
    }
    
    my(@output);
    
    my($tmp);
    $tmp = <$Socket>;
    
    SendLine($Socket, $cmd);
    
    my($first) = 1; my($last);
    while(<$Socket>) {
	if($first) { $last = substr($_, 0, 3); }
	s/\r//g;
	if(substr($_, 3, 1) eq "-") {
	    s/^$last-//;
	    push(@output, $_);
	} else {
	    s/^$last //;
	    push(@output, $_);
	    last;
	}
    }
    SendLine($Socket, "quit\r\n");
    close($Socket);
    return(@output);
}


sub SendLine {
    my($Socket, $cmd) = @_;
    my($result) = send($Socket, $cmd, 0);
    if($result != length($cmd)) {
	HTMLError("Error sending command to host $CONFIG{VDRHOST}\n");
    }
}


sub EPGTree {
    my(@DATA);
    for(SVDRPSend("lste\r\n")) { push(@DATA, $_); }

    my($i); my(@events); undef(%EPG);
    for($i = 0; $i <= $#DATA; $i++) {
	$_ = $DATA[$i];
	chomp;
	if(/^C ([^ ]+) *(.*)/) {
	    undef(@events);
	    my($number, $progname) = ($1, $2);
	    for(; $i <= $#DATA; $i++) {
		$_ = $DATA[$i];
		if(/^E (.*) (.*) (.*)/) {
		    my($time, $duration) = ($2, $3);
		    my($title, $subtitle, $description);
		    for(; $i <= $#DATA; $i++) {
			$_ = $DATA[$i];
			if(/^T (.*)/) { $title = $1 }
			if(/^S (.*)/) { $subtitle = $1 }
			if(/^D (.*)/) { $description = $1 }
			if(/^e/) {
			    push(@events, {
				progname    => $progname,
				number      => $number,
				time        => $time,
				duration    => $duration,
				title       => $title,
				subtitle    => $subtitle,
				description => $description
			    });
			    last;
			}
		    }
		} elsif(/^c/) {
		    my($last) = 0; my(@temp);
		    for(sort({ $a->{time} <=> $b->{time} } @events)) {
			next if($last == $_->{time});
			push(@temp, $_);
			$last = $_->{time};
		    }
		    $EPG{$number} = [ @temp ];
		    last;
		}
	    }
	}
    }
}


sub UptoDate {
    if((time - $LOCALLASTGET) >= $CONFIG{CACHE_TIMEOUT}) {
	EPGTree();
	undef(@CACHE_CHAN);
	for(SVDRPSend("lstc\r\n")) { push(@CACHE_CHAN, $_); }
	$LOCALLASTGET = time;
    }
    return(0);
}


sub GetChannelDesc {
    my(%hash);
    for(@CACHE_CHAN) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	if(length($desc) > 12) {
	    $desc = substr($desc, 0, 12);
	    $desc .= "...",
	}
	$hash{$id} = $desc;
    }
    return(%hash);
}


sub GetChannelID {
    my($sid) = $_[0];
    for(@CACHE_CHAN) {
	chomp;
	my($id, $temp) = split(/ /, $_, 2);	
	my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	if($id == $sid) {
	    return($nnumber);
	}
    }
}


sub EURL {
    my($text) = @_;
    $text =~ s/\&/ampersandonlyencodedforie/g;
    $text =~ s/([^0-9a-zA-Z])/sprintf("%%%2.2x", ord($1))/ge;
    return($text);
}


sub HTMLError {
    my($error) = @_;   
    my($template) = HTML::Template->new(
	filename => "$TEMPLATEDIR/$CONFIG{LANG}_error.html");
    $template->param(error => $error);
    SendHeader("200", "text/html", $template->output);
    return;
}


sub GetHeaderTime {
    my(@weekdays) = ("Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat", "Sun");
    my(@months) = ("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", 
	"Sep", "Oct", "Nov", "Dec");
	       
    my($sec,$min,$hour,$day,$month,$year,$wday,$yday,$isdst) = gmtime(time);
    return sprintf "%s, %s %s %s  %02d:%02d:%02d GMT", $weekdays[$wday], 
	$day, $months[$month], $year, $hour, $min, $sec;
}


sub SendHeader {
    my($Status, $ContentType, $Data) = @_;
    print($Client "HTTP/1.0 $Status\r\n");
    print($Client "Date: ", GetHeaderTime(), "\r\n");
    print($Client "Server: $SERVERVERSION\r\n");
    print($Client "Content-Length: ", length($Data), "\r\n") if($Data);
    print($Client "Connection: close\r\n");
    print($Client "Content-type: $ContentType\r\n\r\n") if($ContentType);
    print($Client $Data) if($Data);
}


sub SendForwardingHeader {
    my($Location) = @_;
    print($Client "HTTP/1.0 302 Found\r\n");
    print($Client "Date: ", GetHeaderTime(), "\r\n");
    print($Client "Server: $SERVERVERSION\r\n");
    print($Client "Connection: close\r\n");
    print($Client "Location: $Location\r\n");
    print($Client "Content-type: text/html\r\n\r\n");
}


sub SendNoAuthHeader {
    my($Status) = @_;
    my($Data) = start_html("Authorization Required") . h1("Authorization Required") . p("This server could not verify that you are authorized to access the document requested. Either you supplied the wrong credentials (e.g., bad password), or your browser doesn't understand how to supply the credentials required.") . hr . end_html;
    print($Client "HTTP/1.0 $Status\r\n");
    print($Client "Date: ", GetHeaderTime(), "\r\n");
    print($Client "Server: $SERVERVERSION\r\n");
    print($Client "WWW-Authenticate: Basic realm=\"Privat\"\r\n");
    print($Client "Content-Length: ", length($Data), "\r\n") if($Data);
    print($Client "Connection: close\r\n");
    print($Client "Content-type: text/html\r\n\r\n");
    print($Client $Data);
}


sub SendFile {
    sub FileError {
	my($et) = start_html($_[0]) . h1($_[1]) . $_[2] . p . hr . end_html;
	SendHeader("$_[0] $_[1]", "text/html", $et);
    }
    my($File) = @_;
    my($buf, $temp);
    $File =~ s/^\///;
    if(-e $BASENAME . $File) {
	if(-r $BASENAME . $File) {
	    open(F, $BASENAME . $File);
	    while(<F>) {
		$buf .= $_;
	    }
	    close(F);
	    $temp = $File;
	    $temp =~ /([A-Za-z0-9]+)\.([A-Za-z0-9]+)/;
	    if(!$mimehash{$2}) { die("can't find mime-type \'$2\'\n"); }
	    SendHeader("200", $mimehash{$2}, $buf);
	} else {
	    FileError("403", "Forbidden", "You don't have permission to" .
		" access /$File on this server.");
	}
    } else {
	FileError("404", "Not Found", "The requested URL /$File was not" .
	    " found on this server.");	
    }
}

#############################################################################
#############################################################################
sub do_something {

    my($q) = CGI->new(@_);

    #
    if($q->param('aktion') eq "prog_detail") {
	my($template) = HTML::Template->new(filename => 
	    "$TEMPLATEDIR/$CONFIG{LANG}_prog_detail.html");
	my($titel) = $q->param('titel');
	$titel =~ s/ampersandonlyencodedforie/\&/g;
	my($text) = $q->param('text');
	$text =~ s/ampersandonlyencodedforie/\&/g;
	$template->param(
	    titel => $titel ? $titel : undef, 
	    text  => $text ? $text : undef,
	);
	SendHeader("200", "text/html", $template->output); 
    }


    #
    if($q->param('aktion') eq "prog_list") {

	return if(UptoDate());
	
	if(!$q->param('number')) {
	    my($desc, $number);
	    for(@CACHE_CHAN) {
		chomp;
		my($id, $temp) = split(/ /, $_, 2);	
		($desc, $number) = (split(/\:/, $temp))[0,9];
		last;
	    }
	    SendForwardingHeader(
		"$MyURL?aktion=prog_list&number=$number");
	    return;
	}  
	
	my(@show, $progname); my($toggle) = 1;
	my($number) = $q->param('number');
	for(@{$EPG{$number}}) {
	    push(@show, {
		ssse        => $_->{time},
		emit        => strftime("%H:%M", localtime($_->{time})),
		duration    => strftime("%H:%M", localtime($_->{time} + $_->{duration})),
		title       => $_->{title},
		subtitle    => $_->{subtitle},
		recurl      => $MyURL . "?aktion=timer_new_form&active=1&chan=" . 
		    $_->{number} . 
		    "&start=$_->{time}" . "&end=" . ($_->{time} + $_->{duration}) . 
		    "&prio=99&lft=99&name=" . EURL($_->{title}) . "&zpuf=1",
		infurl      => $_->{description} ? $MyURL . "?aktion=prog_detail&titel=" 
		    . EURL($_->{title}) . "&text=" . EURL($_->{description}) : undef,
		toggle      => $toggle,
	    });
	    if($toggle) { $toggle = 0; } else { $toggle = 1; }
	    $progname = $_->{progname};
	}
	
       
	#
	my(@chans);
	for(@CACHE_CHAN) {
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    my($desc, $nnumber) = (split(/\:/, $temp))[0,9];
	    if(length($desc) > 12) {
		$desc = substr($desc, 0, 12);
		$desc .= "...",
	    }
	    push(@chans, {
		name => $desc,
		number => $nnumber,
		cur  => ($number == $nnumber) ? 1 : 0,
	    });
	}

	# 
	my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/$CONFIG{LANG}_prog_list.html");
	$template->param(
	    url      => $MyURL,
	    loop     => \@show,
	    chanloop => \@chans,
	    progname => $progname
	);
	SendHeader("200", "text/html", $template->output);
    }

    
    #
    if($q->param('aktion') eq "timer_list") {
	
	return if(UptoDate());
    
	my(@timers);
	my(%chandesc) = GetChannelDesc();
	for(SVDRPSend("lstt\n")) {
	    last if(/^No timers defined/);
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);
	    my($active, $prgnum, $dor, $start, $stop, $prio, $lft, $name, $summary) = split(/\:/, $temp, 9);
	    
	    my($startsse);
	    if(length($dor) == 7) {
		$startsse = timelocal(undef, substr($start, 2, 2), 
		    substr($start, 0, 2), strftime("%d", localtime(time)), 
		    strftime("%m", localtime(time)),
		    strftime("%Y", localtime(time)));
	    } else {
		$startsse = timelocal(undef, substr($start, 2, 2), 
		    substr($start, 0, 2), $dor, strftime("%m", localtime(time)),
		    strftime("%Y", localtime(time)));
	    }
	    
	    my($endsse) = timelocal(undef, substr($stop, 2, 2), 
		substr($stop, 0, 2), strftime("%d", localtime(time)),
		strftime("%m", localtime(time)),
		strftime("%Y", localtime(time)));
	     
	    push(@timers, {
		active => $active ? "Yes" : "No",
		prgnum => $chandesc{$prgnum},
		dor    => $dor,
		start  => substr($start, 0, 2) . ":" . substr($start, 2, 2),
		stop   => substr($stop, 0, 2) . ":" . substr($stop, 2, 2),
		prio   => $prio,
		lft    => $lft,
		name   => $name,
		sum    => $summary,
		delurl => $MyURL . "?aktion=timer_delete&id=$id",
		modurl => $MyURL . "?aktion=timer_new_form&reedit=$id&chan="
		    . GetChannelID($prgnum) . "&active=" . $active . 
		    "&start=$startsse" . "&end=$endsse" . 
		    "&prio=$prio&lft=$lft&dor=$dor&name=" . 
		    EURL($name) . "&summary=" . EURL($summary),
	    });
	}

	my($template) = HTML::Template->new(
	    filename => "$TEMPLATEDIR/$CONFIG{LANG}_timer_list.html"
	);
	$template->param(
	    loop => \@timers,
	    nturl  => $MyURL . "?aktion=timer_new_form",
	);
	SendHeader("200", "text/html", $template->output);
    }


    if($q->param('aktion') eq "timer_new_form") {

	#
	my(@chans);
	for(@CACHE_CHAN) {
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	    next if(!$vpid); # VDR can't record Radio program's
	    next if(!$nnumber); # Whats that? Go away!
	    if(length($desc) > 12) {
		$desc = substr($desc, 0, 12);
		$desc .= "...",
	    }
	    push(@chans, {
		name => $desc,
		id   => $id,
		cur  => $q->param('chan') ?
		    (($nnumber == $q->param('chan')) ? 1 : 0) : undef,
	    });
	}
	
	my($starttime) = $q->param('start');
	my($endtime) = $q->param('end');
	
	if($q->param('zpuf')) {
	    $starttime -= ($CONFIG{ZPUFA} * 60);
	    $endtime += ($CONFIG{ZPUFE} * 60);
	}    
	    
	my($template) = HTML::Template->new(
	    filename => "$TEMPLATEDIR/$CONFIG{LANG}_timer_new.html"
	);
	$template->param(
	    titel  => "Neuer Timer",
	    url    => $MyURL,
	    chans  => \@chans,
	    active => $q->param('active') ? $q->param('active') : 0,
	    dor    => (length($q->param('dor')) == 7) ? $q->param('dor') : strftime("%d", localtime($starttime)),
	    starth => (strftime("%H", localtime($starttime)) ne "") ? strftime("%H", localtime($starttime)) : undef,
	    startm => (strftime("%M", localtime($starttime)) ne "") ? strftime("%M", localtime($starttime)) : undef,
	    endh   => (strftime("%H", localtime($endtime)) ne "") ? strftime("%H", localtime($endtime)) : undef,
	    endm   => (strftime("%M", localtime($endtime)) ne "") ? strftime("%M", localtime($endtime)) : undef,
	    prio   => $q->param('prio') ? $q->param('prio') : 99,
	    lft    => $q->param('lft') ? $q->param('lft') : 99,
	    title  => $q->param('name') ? $q->param('name') : undef,
	    summary=> $q->param('summary') ? $q->param('summary') : undef,
	    reedit => $q->param('reedit') ? $q->param('reedit') : 0
	);
	SendHeader("200", "text/html", $template->output);
    }


    if($q->param('aktion') eq "timer_add") {
	
	my($start, $end, $prio, $lft, $title);

	#
	my($dor) = $q->param('dor');
	
	#
	if($q->param('starth') =~ /\d+/ && $q->param('starth') < 24 && $q->param('starth') >= 0) {
	    if($q->param('startm') =~ /\d+/ && $q->param('startm') < 60 && $q->param('starth') >= 0) {
		$start = $q->param('starth') . $q->param('startm');
	    } else {
		HTMLError("Error: Bitte Startzeit richtig eingeben!");
	    }
	} else { 
	    HTMLError("Error: Bitte Startzeit richtig eingeben!");
	}
	
	#
	if($q->param('endh') =~ /\d+/ && $q->param('endh') < 24 && $q->param('endh') >= 0) {
	    if($q->param('endm') =~ /\d+/ && $q->param('endm') < 60 && $q->param('endm') >= 0) {
		$end = $q->param('endh') . $q->param('endm');
	    } else {
		HTMLError("Error: Bitte Endzeit richtig eingeben!");
	    }
	} else { 
	    HTMLError("Error: Bitte Endzeit richtig eingeben!");
	}

	
	#
	if($q->param('prio') =~ /\d\d/) {
	    $prio = $q->param('prio');
	} else {
	    HTMLError("Error: Bitte Prioritaet richtig eingeben!");
	}

	#
	if($q->param('lft') =~ /\d\d/) {
	    $lft = $q->param('lft');
	} else {
	    HTMLError("Error: Bitte Lifetime richtig eingeben!");
	}
	
	#
	if($q->param('title')) {
	    $title = $q->param('title');
	    $title =~ s/\://g;
	} else {
	    HTMLError("Error: Bitte Titel eingeben!");
	}
	
	if(!$q->param('reedit')) {
	    SVDRPSend("newt " . $q->param('active') . ":" . $q->param('chans')
		. ":" . $dor . ":" . $start . ":" . $end  . ":" . $prio . ":"
		. $lft . ":" . $title . ":" . $q->param('summary') . "\n");
	} else {
	    SVDRPSend("modt " . $q->param('reedit') . " " . $q->param('active')
		. ":" . $q->param('chans') . ":" . $dor . ":" . $start . ":"
		. $end  . ":" . $prio . ":" . $lft . ":" . $title . ":" .
		$q->param('summary') . "\n");
	}
	
	SendForwardingHeader("$MyURL?aktion=timer_list");
    }

    
    #
    if($q->param('aktion') eq "timer_delete") {
	my($id) = $q->param('id');

	my($result) = SVDRPSend("delt $id\r\n");
	if($result =~ /Timer "$id" is recording/) {
	    SVDRPSend("modt $id off\r\n");
	}
	SendForwardingHeader("$MyURL?aktion=timer_list");
    }

    
    #
    if($q->param('aktion') eq "prog_summary") {

	return if(UptoDate());
	
	my($eventtime);
	
	if($q->param('time') ne "") {
	    my($hour, $minute);
	    if($q->param('time') =~ /(\d{1,2})(\D?)(\d{1,2})/) {
		if(length($1) == 1 && length($3) == 1 && !$2) {
		    $hour = $1 . $3;
		} else {
		    ($hour, $minute) = ($1, $3);
		}
		
	    } elsif($q->param('time') =~ /\d/) {
		$hour = $q->param('time');
	    }
	    #if(!$hour && !$minute) {
	    #	$hour = strftime("%H", localtime(time));
	    #	$minute = strftime("%M", localtime(time));
	    #}
	    
	    if($hour <= strftime("%H", localtime(time)) && $minute < strftime("%M", localtime(time))) {
		$eventtime = timelocal(undef, $minute, $hour, strftime("%d", 
		    localtime(time + 86400)), 
		    (strftime("%m", localtime(time + 86400)) - 1), 
		    strftime("%Y", localtime(time))) + 1;
	    } else {
		$eventtime = timelocal(undef, $minute, $hour, strftime("%d", 
		    localtime(time)), (strftime("%m", localtime(time)) - 1), 
		    strftime("%Y", localtime(time))) + 1;
	    } 
	} else {
	    $eventtime = time;
	}
		
	#
	my(@chans);
	for(@CACHE_CHAN) {
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	    (my($radio) = 1) if(!$vpid);
	    if(length($desc) > 12) {
		$desc = substr($desc, 0, 12);
		$desc .= "...",
	    }
	    push(@chans, {
		name   => $desc,
		radio  => $radio,
		url    => $MyURL . "?aktion=prog_list&number=$nnumber",
		number => $nnumber
	    });
	}
	
	my(@temp);
	for(keys(%EPG)) {
	    for(@{$EPG{$_}}) {
		if($CONFIG{PCHANL}) {
		    my($f) = 0;
		    for my $n (split(/\,/, $CONFIG{CHANS})) {
			($f = 1) if($n eq $_->{number});
		    }
		    next if(!$f);
		}
		next if($eventtime > ($_->{time} + $_->{duration}));
		push(@temp,  {
		    sse        => $_->{time},
		    prog       => $_->{progname},
		    date       => strftime("%d.%m.%Y", localtime($_->{time})),
		    emit       => strftime("%H:%M", localtime($_->{time})),
		    duration   => strftime("%H:%M", localtime($_->{time} + $_->{duration})),
		    title      => $_->{title} ? $_->{title} : undef,
		    subtitle   => $_->{subtitle} ? ((length($_->{subtitle}) > 30) ? 
			substr($_->{subtitle}, 0, 30) . "..." : $_->{subtitle}) : undef,
		    summary    => $_->{description} ? substr($_->{description}, 0, 120) . "... " : undef,
		    infurl     => $_->{description} ? $MyURL . "?aktion=prog_detail&titel=" 
			. EURL($_->{progname} . ' | ' . strftime("%H:%M", localtime($_->{time})) 
			. '-' . strftime("%H:%M", localtime($_->{time} + $_->{duration})) . ' | '
			. $_->{title}) . "&text=" . EURL($_->{description}) : undef,
		    recurl     => $MyURL . "?aktion=timer_new_form&active=1&chan="
			. $_->{number} . "&start=$_->{time}&end=" .
			($_->{time} + $_->{duration}) . "&prio=99&lft=99&name="
			. EURL($_->{title}) . "&zpuf=1",
		    proglink   => $MyURL . "?aktion=prog_list&number=" . $_->{number},
		    row0       => undef,
		    row1       => undef,
		
		});
		last;
	    }
	}
	my($rows); my($maxrows) = 2; my(@show); my($toggle) = 0; 
	for(sort({ lc($a->{prog}) cmp lc($b->{prog}) } @temp)) {
	    ($rows == 0) ? ($_->{row0} = 1) : ($_->{row0} = 0);
	    ($rows == $maxrows) ? ($_->{row1} = 1) : ($_->{row1} = 0);
	    push(@show, $_);
	    if($rows < $maxrows) {
		$rows++;
	    } else {
		$rows = 0;
	    }
	}
	if($rows) {
	    for(; $rows <= $maxrows; $rows++) {
		if($rows == $maxrows) {	
		    push(@show, { row1 => 1 });
		} else {
		    push(@show, { row0 => undef });
		}
	    }
	}
       

	# 
	my($template) = HTML::Template->new(
	    filename => "$TEMPLATEDIR/$CONFIG{LANG}_prog_summary.html"
	);
	$template->param(
	    rows   => \@show,
	    now    => strftime("%H:%M", localtime($eventtime)),
	    nowurl => $MyURL . "?aktion=prog_summary",
	    url    => $MyURL
	);
	SendHeader("200", "text/html", $template->output);
    }


    #
    if($q->param('aktion') eq "rec_list") {
	
	my(@temp);
	
	for(SVDRPSend("lstr\r\n")) {
	    chomp;
	    if(/^Command unrecognized\: \"lstr\"/) {
		HTMLError("Recording's listing not possible, please apply the"
		    . "patch 'svdrp-0.xy.patch' delivered with this package!");
		return;
	    }
	    my($new);
	    my($id, $date, $time, $name) = split(/ +/, $_, 4);
	    
	    if(substr($time, 5, 1) eq "*") {
		$new = 1;
		$time =~ s/\*//;
	    }

	    push(@temp, {
		date   => $date,
		time   => $time,
		name   => $name,
		new    => $new,
		delurl => $MyURL . "?aktion=rec_delete&id=$id",
		infurl => $MyURL . "?aktion=rec_detail&id=$id"
	    });
	}
	my(@recordings) = sort({ $a->{date} <=> $b->{date} } @temp);
    
	my($template) = HTML::Template->new(
	    filename => "$TEMPLATEDIR/$CONFIG{LANG}_rec_list.html"
	);
	$template->param(recloop => \@recordings);
	SendHeader("200", "text/html", $template->output);

    }


    #
    if($q->param('aktion') eq "rec_delete") {
	my($id) = $q->param('id');
	SVDRPSend("delr $id\r\n");
	SendForwardingHeader("$MyURL?aktion=rec_list");
    }


    #
    if($q->param('aktion') eq "rec_detail") {
	my($id) = $q->param('id');
	my($text) = SVDRPSend("lstr $id\r\n");
	my($template) = HTML::Template->new(filename => "$TEMPLATEDIR/$CONFIG{LANG}_prog_detail.html");
	my(@array) = split(/\|/, $text);
	my($titel); undef($text);
	for(my($i) = 0; $i <= $#array; $i++) {
	    if($i == 0) { 
		$titel = $array[$i];
	    } else {
		$text .= $array[$i];
	    }
	}
	$template->param(
	    text  => $text ? $text : "",
	    titel => $titel
	);
	SendHeader("200", "text/html", $template->output);	
    }


    #
    if($q->param('aktion') eq "conf_list") {
	
	return if(UptoDate());
	
	my(@lang, @allchans, @selchans, @temp);
	
	if($q->param('submit') eq ">>>>>") {
	    if($CONFIG{CHANS}) {
		for my $c ($q->param('allchans')) {
		    my($found) = 0;
		    for(split(/\,/, $CONFIG{CHANS})) {
			if($_ eq $c) {
			    $found = 1;
			}
		    }
		    if(!$found) {	
			$CONFIG{CHANS} .= "," . $c;
		    }
		}
	    } else {
		$CONFIG{CHANS} = $q->param('allchans');
	    }
	    $CONFIG{PCHANL} = 1;
	}
	if($q->param('submit') eq "<<<<<") {
	    if($CONFIG{CHANS}) {
		for my $c ($q->param('selchans')) {
		    my($temp);
		    for(split(/\,/, $CONFIG{CHANS})) {
			if($_ ne $c) {
			    if($temp) {
				$temp .= ",$_";
			    } else {
				$temp = $_;
			    }
			}
		    }
		    $CONFIG{CHANS} = $temp;
		}
	    }
	    $CONFIG{PCHANL} = 1;
	}
	if($q->param('submit') eq "Einstellungen Speichern") {
	    for($q->param) {
		if(/[A-Z]+/) {
		    $CONFIG{$_} = $q->param($_);
		}
	    }
	    open(CONF, ">$CONFFILE");
	    for(keys(%CONFIG)) {
		print(CONF "$_ = $CONFIG{$_}\n");
	    }
	    close(CONF);
	}
	if($q->param('submit') eq "Anwenden") {
	    for($q->param) {
		if(/[A-Z]+/) {
		    $CONFIG{$_} = $q->param($_);
		}
	    }
	}
	    
	
	#	
	for(<$TEMPLATEDIR/*>) {
	    s/.*\///g;
	    if(/^(\w\w)_.*/) {
		my($found) = 0;
		for(@lang) { ($found = 1) if($_->{name} eq $1); }
		if(!$found) { 
		    push(@lang, {
			name   => $1,
			aklang => ($CONFIG{LANG} eq $1) ? 1 : 0,
		    }); 
		}
	    }
	}
	
	#
	for(@CACHE_CHAN) {
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	    (my($radio) = 1) if(!$vpid);
	    if(length($desc) > 12) {
		$desc = substr($desc, 0, 12);
		$desc .= "...",
	    }
	    push(@allchans, {
		name => $desc,
		number => $nnumber
	    });
	}

	#
	for(@CACHE_CHAN) {
	    my($found) = 0;
	    chomp;
	    my($id, $temp) = split(/ /, $_, 2);	
	    my($desc, $nnumber, $vpid) = (split(/\:/, $temp))[0,9,5];
	    (my($radio) = 1) if(!$vpid);
	    for(split(/\,/, $CONFIG{CHANS})) {
		if($_ eq $nnumber) {
		    $found = 1;
		}
	    }
	    next if(!$found); 
	    if(length($desc) > 12) {
		$desc = substr($desc, 0, 12);
		$desc .= "...",
	    }
	    push(@selchans, {
		name => $desc,
		number => $nnumber
	    });
	
	}
	
	my($template) = HTML::Template->new(
	    filename => "$TEMPLATEDIR/$CONFIG{LANG}_config.html"
	);
	$template->param(
	    VDRHOST       => $CONFIG{VDRHOST},
	    VDRPORT       => $CONFIG{VDRPORT},
	    SERVERPORT    => $CONFIG{SERVERPORT},
	    CACHE_TIMEOUT => $CONFIG{CACHE_TIMEOUT},
	    USER          => $CONFIG{USER},
	    PASS          => $CONFIG{PASS},
	    PCHANL        => $CONFIG{PCHANL},
	    ZPUFA         => $CONFIG{ZPUFA},
	    ZPUFE         => $CONFIG{ZPUFE},
	    LOGGING       => $CONFIG{LOGGING},
	    LOGFILE       => $CONFIG{LOGFILE},
	    LANG          => \@lang,
	    ALLCHANS      => \@allchans,
	    SELCHANS      => \@selchans,
	    url           => $MyURL,
	);
	SendHeader("200", "text/html", $template->output);
    }
}

# EOF
